/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors;

import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.DomainValue;
import it.eng.spago4q.bo.InterfaceField;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QFKDAO;

import java.util.HashMap;
import java.util.List;

public class DomainValueCacheCreator {

	private HashMap<String, HashMap<String, Integer>> cache;

	public DomainValueCacheCreator(final Integer interfaceTypeId)
			throws EMFUserError {
		// get interfaceField
		IES4QFKDAO eInterfaceFieldDAO = (IES4QFKDAO) DAOFactory
				.getDAO("EInterfaceFieldDAO");

		List<InterfaceField> interfaceFieldList = (List<InterfaceField>) eInterfaceFieldDAO
				.loadObjectList(interfaceTypeId, null, null);

		if (interfaceFieldList != null && !(interfaceFieldList.isEmpty())) {
			// get DomainValueDAO
			cache = new HashMap<String, HashMap<String, Integer>>();
			IES4QFKDAO eDomainValueDAO = (IES4QFKDAO) DAOFactory
					.getDAO("EDomainValueDAO");

			for (InterfaceField interfaceField : interfaceFieldList) {
				List domainValueList = eDomainValueDAO.loadObjectList(
						interfaceField.getId(), null, null);
				if (domainValueList != null && !(domainValueList.isEmpty())) {
					HashMap<String, Integer> domainValueHashMap = 
						createDomainValueHashMap((List<DomainValue>) domainValueList);
					cache.put(interfaceField.getName(), domainValueHashMap);
				}
			}
		}
	}

	public Integer getDomainValueId(final String columnName,
			final String valueName) {
		Integer toReturn = null;
		HashMap<String, Integer> columnHashMap = cache.get(columnName);
		if (columnHashMap != null) {
			toReturn = columnHashMap.get(valueName);
		}
		return toReturn;
	}

	public boolean hasDomainValue(final String columnName) {
		boolean toReturn = false;
		if (cache.get(columnName) != null && !(cache.isEmpty())) {
			toReturn = true;
		}
		return toReturn;
	}

	private HashMap<String, Integer> createDomainValueHashMap(
			final List<DomainValue> domainValueList) {
		HashMap<String, Integer> toReturn = new HashMap<String, Integer>();
		for (DomainValue domainValue : domainValueList) {
			toReturn.put(domainValue.getValue(), domainValue.getId());
		}
		return toReturn;
	}
}
