/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors;

import org.apache.log4j.Logger;

import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.EProcess;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QDAO;

public final class OperationCoordinatorFactory {
	
	private OperationCoordinatorFactory() {}

	private static final String EPROCESS_DAO = "EProcessDAO";

	private static Logger logger = Logger
			.getLogger(OperationCoordinatorFactory.class);

	public static OperationCoordinatorInterface create(final Integer processId, final String extractionDir)
			throws EMFUserError {
		IES4QDAO eProcessDAO = (IES4QDAO) DAOFactory.getDAO(EPROCESS_DAO);

		EProcess eProcess = (EProcess) eProcessDAO.loadObjectById(processId);

		String coordinatorClassName = eProcess.getCoordinatorClass();

		OperationCoordinatorInterface coordinator = null;

		try {
			coordinator = (OperationCoordinatorInterface) Class.forName(
					coordinatorClassName).newInstance();
			coordinator.init(eProcess, extractionDir);
			
			//TODO cambiare messaggi errori
		} catch (InstantiationException e) {
			logger.error("Error while Instantiate class "
					+ coordinatorClassName, e);
			throw new EMFUserError(EMFErrorSeverity.ERROR, 101);
		} catch (IllegalAccessException e) {
			logger.error("Error while Instantiate class "
					+ coordinatorClassName, e);
			throw new EMFUserError(EMFErrorSeverity.ERROR, 101);
		} catch (ClassNotFoundException e) {
			logger.error("Error while Instantiate class "
					+ coordinatorClassName, e);
			throw new EMFUserError(EMFErrorSeverity.ERROR, 101);
		}

		return coordinator;
	}

}
