/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors.scheduler;

import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.extractors.scheduler.job.ProcessExtractionJob;

import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;

import org.quartz.CronTrigger;
import org.quartz.JobDetail;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.quartz.impl.StdSchedulerFactory;

public final class SchedulerSupplier {
	
	private SchedulerSupplier() {}

	private static JobDetail getExtractionJob(final Integer processId, final String archiveDir) {
		JobDetail jobDetail = new JobDetail("PROCESS_" + processId,
				"EXTRACTION_PROCESS_GROUP", ProcessExtractionJob.class);

		jobDetail.getJobDataMap().put("E_PROCESS_ID", processId);
		jobDetail.getJobDataMap().put("ARCHIVE_DIR", archiveDir);
		jobDetail.setDurability(true);
		jobDetail.setVolatility(false);
		/*
		 * RequestsRecovery - if a job "requests recovery", and it is executing
		 * during the time of a 'hard shutdown' of the scheduler (i.e. the
		 * process it is running within crashes, or the machine is shut off),
		 * then it is re-executed when the scheduler is started again. In this
		 * case, the JobExecutionContext.isRecovering() method will return true.
		 */
		jobDetail.setRequestsRecovery(true);

		return jobDetail;
	}

	private static Trigger getExtractionCronTrigger(final String cronString,
			final Integer processId) throws ParseException {
		CronTrigger trigger = new CronTrigger();
		trigger.setName("TRIGGER_" + processId);
		trigger.setGroup("TRIGGER_PROCESS_GROUP");
		trigger.setCronExpression(cronString);
		// trigger.setMisfireInstruction(CronTrigger.MISFIRE_INSTRUCTION_SMART_POLICY);
		Date startTime = Calendar.getInstance().getTime();
		trigger.setStartTime(startTime);
		return trigger;

	}

	public static void addExtractionProcess(final Integer processId, final String cronString, final String archiveDir)
			throws EMFUserError {
		try {

			Scheduler scheduler = StdSchedulerFactory.getDefaultScheduler();
			// set Job
			JobDetail jobDetail = getExtractionJob(processId, archiveDir);

			// set Trigger
			Trigger trigger = getExtractionCronTrigger(cronString, processId);
			scheduler.scheduleJob(jobDetail, trigger);

		} catch (SchedulerException se) {
			// TODO Auto-generated catch block
			se.printStackTrace();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public static void removeExtractionProcess(final Integer processId) {
		try {
			Scheduler scheduler = StdSchedulerFactory.getDefaultScheduler();
			scheduler.deleteJob("PROCESS_" + processId,
					"EXTRACTION_PROCESS_GROUP");
		} catch (SchedulerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
